function widget:GetInfo()
	return {
		name      = "Ore Area Reclaim",
		desc      = "Area reclaims ore or not depending on the reclaim radius set in Ore Reclaim Radius Control",
		author    = "zerver",
		date      = "Apr 2012",
		license   = "Public Domain",
		layer     = 0,
		enabled   = true
	}
end

local spGetMyTeamID = Spring.GetMyTeamID
local spGetTeamRulesParam = Spring.GetTeamRulesParam
local spGiveOrderToUnit = Spring.GiveOrderToUnit
local spGetSelectedUnits = Spring.GetSelectedUnits
local spGetDefaultCommand = Spring.GetDefaultCommand
local spIsAboveMiniMap = Spring.IsAboveMiniMap
local spGetTimer = Spring.GetTimer
local spDiffTimers = Spring.DiffTimers
local spGetActiveCommand = Spring.GetActiveCommand
local spGetKeyState = Spring.GetKeyState
local spGetMouseState = Spring.GetMouseState
local spTraceScreenRay = Spring.TraceScreenRay
local spGetViewGeometry = Spring.GetViewGeometry

local glText = gl.Text
local glGetTextWidth = gl.GetTextWidth
local glPushMatrix = gl.PushMatrix
local glPopMatrix = gl.PopMatrix
local glPushAttrib = gl.PushAttrib
local glPopAttrib = gl.PopAttrib
local glDepthTest = gl.DepthTest
local glTranslate = gl.Translate
local glBillboard = gl.Billboard
local glColor = gl.Color
local glLineWidth = gl.LineWidth
local glDrawGroundCircle = gl.DrawGroundCircle
local glBlending = gl.Blending

local m_floor = math.floor
local m_sqrt = math.sqrt
local m_bits = math.bit_or(GL.CURRENT_BIT, GL.ENABLE_BIT)
local s_format = string.format

local defaultOreRadius = 2.0
local inReclaimCmd = false
local oreSearchRad = Game.extractorRadius
local doSwitchReclaim = false
local start = false
local rangestart = {}
local rangeend = {}
local MiniMapFullProxy = (Spring.GetConfigInt("MiniMapFullProxy", 0) == 1)
local inMiniMap = false
local usingCmd = nil
local usingRMB = false
local form = 20
local xstart,ystart = 0
local oreSearchRad = Game.extractorRadius
local lastSwitch = spGetTimer()
local b1p = false
local b3p = false
local numDraw = 0
local lastulimit = false

function widget:CommandNotify(cmdID, cmdParams, cmdOptions)
	if (cmdID == CMD.RECLAIM) and not(cmdOptions.meta) and not(inReclaimCmd) and #cmdParams == 4 then
		local myTeamID = spGetMyTeamID()
		local oreRadFactor = spGetTeamRulesParam(myTeamID, 'oreRadius')
  		oreRadFactor = (oreRadFactor ~= nil) and oreRadFactor or defaultOreRadius
  		local doSwitch
  		if  doSwitchReclaim ~= nil then
  			doSwitch = doSwitchReclaim
  		else
  			doSwitch = cmdParams[4] <= oreRadFactor * oreSearchRad
  		end
		doSwitchReclaim = nil
		local opt = {}
		if doSwitch then
			if not(cmdOptions.ctrl) then 
				for x,z in pairs(cmdOptions) do if z then opt[#opt + 1] = x end end
				opt[#opt + 1] = "ctrl"
			else
			for x,z in pairs(cmdOptions) do if z and (x ~= "ctrl") then opt[#opt + 1] = x end end
			end
		else
			for x,z in pairs(cmdOptions) do if z then opt[#opt + 1] = x end end
		end
		inReclaimCmd = true
		local units = spGetSelectedUnits()
		for i=1,#units do
			spGiveOrderToUnit(units[i], CMD.RECLAIM, cmdParams, opt)
		end
		inReclaimCmd = false
		return true
	end
	return false
end




function widget:MousePress(mx, my, mButton)
	if mButton == 1 then
		b1p = true
	end
	if mButton == 3 then
		b3p = true
	end
	inMinimap = spIsAboveMiniMap(mx, my)
	if (usingCmd ~= nil) or (inMinimap and not MiniMapFullProxy) then
		return false
	end
        
        local cmd = false
	local _, activeCmdID = spGetActiveCommand()
	if activeCmdID then
		if mButton ~= 1 or activeCmdID ~= CMD.RECLAIM then
			return false
		end
                
		usingCmd = activeCmdID
		usingRMB = false
		cmd = true
	else
		if mButton ~= 3 then
			return false
		end
                
		local _, defaultCmdID = spGetDefaultCommand()
		if defaultCmdID ~= CMD.RECLAIM then
			return false
		end
		
		usingCmd = defaultCmdID
		usingRMB = true
		cmd = true
	end
	
	if cmd and (usingCmd == CMD.RECLAIM) then
		doSwitchReclaim = nil
		numDraw = 0
		local x, y = m_floor(mx), m_floor(my)
		xstart, ystart = x, y
		start = false
		_, rangestart = spTraceScreenRay(x, y, true, inMinimap)
		lastSwitch = spGetTimer()
	end
	
	return false
end

local function CancelReclaimCmd()
	doSwitchReclaim = nil
	usingCmd = nil
	usingRMB = false
	rangestart = {0, _,0}
	rangeend = {0, _,0}
	start = false
	inMiniMap = false
end

function widget:KeyPress(key, mods, isRepeat)
	if key == 27 then
		CancelReclaimCmd()
	end
	return false
end

function widget:DrawWorld()
	local x, y, b1,b2,b3, b1r, b3r
	if b1p or b3p then
		x, y, b1, b2, b3 = spGetMouseState()
		x, y = m_floor(x), m_floor(y)
		b1r = b1p and not(b1)
		if b1r then
			b1p = false
		end
		b3r = b3p and not(b3)
		if b3r then
			b3p = false
		end
	end

	if (rangestart == nil) or (usingCmd ~= CMD.RECLAIM) then
		return false
	end
	if ((b1r or (b1 and b3r)) and not(usingRMB)) or ((b3r or (b3 and b1r)) and usingRMB) then
		CancelReclaimCmd()
		return false
	end
	local _, activeCmdID = spGetActiveCommand()
	if not(usingRMB) and (activeCmdID ~= CMD.RECLAIM) then
		CancelReclaimCmd()
		return false
	end
	
	if not (start) then
		if ((x < xstart-4) or (xstart+4 < x) or (y < ystart-4) or (ystart+4 < y)) then
			start=true
			lastSwitch = spGetTimer()
		end
	else
		_, rangeend = spTraceScreenRay(x,y,true, inMinimap)
		if(rangeend == nil) then
			return 
		end

		local rdx, rdy = (rangestart[1] - rangeend[1]), (rangestart[3]- rangeend[3])
		local dist = m_sqrt((rdx * rdx) + (rdy * rdy))

		local myTeamID = spGetMyTeamID()
		local oreRadFactor = spGetTeamRulesParam(myTeamID, 'oreRadius')
  		oreRadFactor = (oreRadFactor ~= nil) and oreRadFactor or defaultOreRadius
 
		local ulimit = dist <= oreRadFactor * oreSearchRad
		numDraw = numDraw + 1
		if numDraw > 5 then
			doSwitchReclaim = ulimit
		end
		if ulimit ~= lastulimit then
			lastSwitch = spGetTimer()
			lastulimit = ulimit
		end

		if spGetKeyState(306) then
			ulimit = not(ulimit)
		end
		local msg
		if ulimit then
			msg = "Ore+All"
		else
			msg = "Normal"
		end

		local tog = "(ctrl toggles)"

		if not(spGetKeyState(310)) and oreRadFactor > 0 then
			local diff = spDiffTimers(spGetTimer(), lastSwitch)
			if diff < 1.0 then
				glLineWidth(2.0 * (1.0 - diff))
				glBlending("alpha")
				glColor(1, 1, 1, (1.0 - diff))
				glDrawGroundCircle(rangestart[1], rangestart[2], rangestart[3], oreRadFactor * oreSearchRad, 32)
				glColor(1, 1, 1, 1)
				glLineWidth(1.0)
			end

			glPushAttrib(m_bits)
			glDepthTest(true)
			glPushMatrix()
			
			glPushMatrix()
			glTranslate(rangestart[1], rangestart[2], rangestart[3])
			glBillboard()
			glColor(1.0, 1.0, 1.0, 1.0)
			
			glText(msg, 0, 0, form, "xco")
			if spDiffTimers(spGetTimer(), lastSwitch) > 1.0 then
				glText(tog, 0, -form, form, "xco")
			end
			
			glPopMatrix()
			
			glPopMatrix()
			glPopAttrib()
		end
	end
end
